package main

import "encoding/json"
import "fmt";
import "io/ioutil";
import "log";
import "net/http";
import "os";
import "path";
import "path/filepath";
import "runtime";
import "strings";
import "time";
import dns "github.com/denverdino/aliyungo/dns";


type Config struct {
	AppId string `json:"appid"`;
	AppSecret string `json:"appsecret"`;
	Domain string `json:"domain"`;
	Key string `json:"key"`;
	Loop int `json:"loop"`;
	Sleep int `json:"sleep"`;
	Ipv4 int`json:"ipv4"`;
	Ipv6 int `json:"ipv6"`;
	LogTime int `json:"logtime"`;
	Ipv4Host string `json:"ipv4host"`;
	Ipv6Host string `json:"ipv6host"`;
}

type Database struct {
	Ipv4 string `json:"ipv4"`;
	Ipv4RecordId string `json:"ipv4recordid"`;
	Ipv6 string `json:"ipv6"`;
	Ipv6RecordId string `json:"ipv6recordid"`;
	LastTime string `json:"lasttime"`;
}

func main() {
	sRunPath := GetCurrentAbPath();
	sConfigPath := sRunPath + "/config.json";
	oFileConfig, errc := os.Open(sConfigPath);
	defer oFileConfig.Close();
	sDatabasePath := sRunPath + "/database.json";
	oFileDatabase, errd := os.Open(sDatabasePath);
	defer oFileDatabase.Close();
	if (errc == nil && errd == nil) {
		var conf Config;
		oJsonConfig := json.NewDecoder(oFileConfig);
		errc = oJsonConfig.Decode(&conf);
		var db Database;
		oJsonDatabase := json.NewDecoder(oFileDatabase);
		errd = oJsonDatabase.Decode(&db);
		if (errc == nil && errd == nil) {
			var client *dns.Client;
			client = dns.NewClient(conf.AppId, conf.AppSecret);
			iLoop := conf.Loop;
			iSleep := conf.Sleep;
			if iSleep < 0 {
				iSleep = 60;
			}
			if iLoop < 1 {
				iLoop = 1;
			}
			var  iWrite int;
			iWrite = 0;
			time.LoadLocation("Asia/Shanghai");
			var sIpv4RecordId string;
			if db.Ipv4RecordId != "" {
				sIpv4RecordId = db.Ipv4RecordId;
			} else {
				sIpv4RecordId = getRecordId(client, conf.Domain, conf.Key, "A");
				db.Ipv4RecordId = sIpv4RecordId;
				iWrite = 1;
				//writeToJsonFile(db, sDatabasePath);
			}
			var sIpv6RecordId string;
			if db.Ipv6RecordId != "" {
				sIpv6RecordId = db.Ipv6RecordId;
			} else {
				sIpv6RecordId = getRecordId(client, conf.Domain, conf.Key, "AAAA");
				db.Ipv6RecordId = sIpv6RecordId;
				iWrite = 1;
				//writeToJsonFile(db, sDatabasePath);
			}
			for i:=0; i<iLoop; i++ {
				if conf.Ipv4 == 1 {
					sIpv4 := getIPV4(conf.Ipv4Host);
					if db.Ipv4 != sIpv4 {
						db.Ipv4 = sIpv4;
						iWrite = 1;
						loopWork(client, conf.Domain, sIpv4RecordId, conf.Key, sIpv4, "A");
					}
				}
				if conf.Ipv6 == 1 {
					sIpv6 := getIPV6(conf.Ipv6Host);
					if db.Ipv6 != sIpv6 {
						db.Ipv6 = sIpv6;
						iWrite = 1;
						loopWork(client, conf.Domain, sIpv6RecordId, conf.Key, sIpv6, "AAAA");
					}
				}
				db.LastTime = time.Now().Format(DateFormat("Y-m-d H:i:s"));
				if iWrite == 1 || conf.LogTime == 1 {
					writeToJsonFile(db, sDatabasePath);
					iWrite = 0;
				}
				time.Sleep(time.Duration(iSleep) * time.Second);
			}
		} else {
			fmt.Println("Json decode failed.", errc.Error());
		}
	} else {
		if os.IsNotExist(errc) || os.IsNotExist(errd) {
			fmt.Println("Config file is not Exist: \n" + sConfigPath + "\n" + sDatabasePath);
		} else {
			fmt.Println("Config file error: \n" + sConfigPath + "\n" + sDatabasePath);
		}
	}
}

func loopWork(client *dns.Client, sDomain string, sRecordId string, sKey string, sIp string, sType string) {
	if sIp != "" {
		aData := dns.UpdateDomainRecordArgs {
			RecordId: sRecordId,
			RR:       sKey,
			Value:    sIp,
			Type:     sType,
		}
		_, err := client.UpdateDomainRecord(&aData);
		if err != nil {
			sTime := time.Now().Format(DateFormat("Y-m-d H:i:s"));
			fmt.Println(sTime + " : ", err.Error());
		}
	}
}

func DateFormat(sTimeType string) string {
	sTimeType = strings.Replace(sTimeType, "Y", "2006", -1);
	sTimeType = strings.Replace(sTimeType, "y", "06", -1);
	sTimeType = strings.Replace(sTimeType, "m", "01", -1);
	sTimeType = strings.Replace(sTimeType, "d", "02", -1);
	sTimeType = strings.Replace(sTimeType, "H", "15", -1);
	sTimeType = strings.Replace(sTimeType, "g", "03", -1);
	sTimeType = strings.Replace(sTimeType, "i", "04", -1);
	sTimeType = strings.Replace(sTimeType, "s", "05", -1);
	return sTimeType;
}

// 获取当前执行文件绝对路径
func GetCurrentAbPath() string {
	dir := getCurrentAbPathByExecutable()
	if strings.Contains(dir,getTmpDir())  {
		return getCurrentAbPathByCaller()
	}
	return dir
}

// 获取当前执行文件绝对路径（仅支持 go run）
func getCurrentAbPathByCaller() string {
	var abPath string
	_, filename, _, ok := runtime.Caller(0)
	if ok {
		abPath = path.Dir(filename)
	}
	return abPath
}

// 获取当前执行文件绝对路径（仅支持 go build）
func getCurrentAbPathByExecutable() string {
	exePath, err := os.Executable()
	if err != nil {
		log.Fatal(err)
	}
	res, _ := filepath.EvalSymlinks(filepath.Dir(exePath))
	return res
}

func getIPV4(sIpHost string) string {
	resp, err := http.Get(sIpHost)
	if err != nil {
		return ""
	}
	defer resp.Body.Close()
	content, _ := ioutil.ReadAll(resp.Body)
	return string(content)
}

func getIPV6(sIpHost string) string {
	resp, err := http.Get(sIpHost)
	if err != nil {
		return ""
	}
	defer resp.Body.Close()
	content, _ := ioutil.ReadAll(resp.Body)
	return string(content)
}

func getRecordId(client *dns.Client, sDomain string, sKey string, sType string) string {
	describeArgs := dns.DescribeDomainRecordsArgs {
		DomainName: sDomain,
		RRKeyWord: sKey,
		TypeKeyWord: sType,
	}
	descResponse, err := client.DescribeDomainRecords(&describeArgs);
	if err == nil {
		for _, descRecord := range descResponse.DomainRecords.Record {
			return descRecord.RecordId;
		}
	} else {
		fmt.Println("getRecordId() : Aliyun error.", err.Error());
	}
	return "";
}

// 获取系统临时目录（兼容 go run）
func getTmpDir() string {
	dir := os.Getenv("TEMP")
	if dir == "" {
		dir = os.Getenv("TMP")
	}
	res, _ := filepath.EvalSymlinks(dir)
	return res
}

func writeToJsonFile(db Database, sDatabasePath string) {
	output, _ := json.MarshalIndent(&db,"", "\t");
	ioutil.WriteFile(sDatabasePath, output, 0644);
}
